package com.gmail.sg06.greenrevolution.data;

import java.util.ArrayList;

/**
 * 狩人用のデータクラス．
 * knownlist はguardlistで読み替えて，
 * isBlack等は使わない！
 *
 * @author hasegawa
 *
 */
public class BGCOData{

	private int day_;//COした日にち．してなければ最初に結果を言った日

	private int turn_;//COしたターン．してなければ最初に結果を言ったターン

	private ArrayList<Integer> guardList_;

	private boolean isCO_;

	private boolean isAfterSlide_;//スライドCOしたかどうか．（注）この役職がスライドCO後ならtrue

	private boolean isAlive_;//この人が生きているか否か


	public BGCOData(int day, int turn, boolean isCO) {
		day_ = day;
		turn_ = turn;
		isCO_ = isCO;
		isAlive_ = true;
		setGuardList(new ArrayList<Integer>());
	}

	/**
	 * COした日にち
	 * してなければ最初に結果を言った日
	 * @return 日にち
	 */
	public int getDay() {
		return day_;
	}

	/**
	 * COしたターン数
	 *してなければ最初に結果を言ったターン
	 *
	 * @return
	 */
	public int getTurn() {
		return turn_;
	}

	/**
	 * n 日目の結果！
	 *
	 * @param day 知りたい日にち
	 * @return エージェントindex 守り結果が無い場合は -1
	 */
	public int getAgent(int day) {
		if (day < 1) {//0日目は誰も噛まれないので受け付けない！
			System.err.println("day must be more than 1");
		}

		if (getGuardList().size() < day) {
			return -1;
		}

		return getGuardList().get(day - 1);
	}

	/**
	 * すべての日の分結果を言ったかどうか
	 * 注) その日の最初に呼ぶべきではない！結果を言うのが２ターン目だったり，
	 * 遅れてCOして，まとめて結果を言っている最中かもしれないので，
	 * 使うならtalkが全て終わってから！！
	 * @param day 現在の日にち
	 * @return
	 */
	public boolean isFullResult(int day) {
		if (getGuardList().size() != day - 1) {
			return false;
		}
		return true;
	}

	/**
	 * 狩人結果を追加
	 *
	 * @param agentId
	 *            エージェントのIndex
	 */
	public void add(int targetIdx) {
		getGuardList().add(targetIdx);
	}

	/**
	 * 結果を言った回数
	 *
	 * @return
	 */
	public int getResultNum() {
		return getGuardList().size();
	}

	public ArrayList<Integer> getGuardList() {
		return guardList_;
	}

	public void setGuardList(ArrayList<Integer> guardList) {
		guardList_ = guardList;
	}

	public boolean isAlive() {
		return isAlive_;
	}

	public void setAlive(boolean isAlive) {
		isAlive_ = isAlive;
	}

	public boolean isAfterSlide() {
		return isAfterSlide_;
	}

	public void setAfterSlide(boolean isAfterSlide) {
		isAfterSlide_ = isAfterSlide;
	}

}
