package com.gmail.sg06.greenrevolution.data;

import java.util.ArrayList;
import java.util.HashMap;

/**
 * SeerとMediam候補のデータ格納クラス
 * @author hasegawa
 *
 */
public abstract class COData {

	private int day_;//COした日にち．してなければ最初に結果を言った日

	private int turn_;//COしたターン．してなければ最初に結果を言ったターン

	private ArrayList<Integer> dayList_;		// 結果を言った日にちのリスト

	private ArrayList<Integer> knownList_;		// 結果を言ったエージェントのリスト

	private ArrayList<Boolean> isBlackList_;		// 言った結果. 黒ならtrue, 白ならfalse

	private boolean isCO_;

	private boolean isAfterSlide_;//スライドCOしたかどうか．（注）この役職がスライドCO後ならtrue

	private boolean isAlive_;//この人が生きているか否か. スライドCOした場合もこちらをfalseにする．

	public COData(int day, int turn, boolean isCO) {
		day_ = day;
		turn_ = turn;
		isCO_ = isCO;
		isAlive_ = true;
		setDayList(new ArrayList<Integer>());
		setKnownList(new ArrayList<Integer>());
		setIsBlackList(new ArrayList<Boolean>());
	}

	/**
	 * COした日にち
	 * してなければ最初に結果を言った日
	 * @return 日にち
	 */
	public int getDay() {
		return day_;
	}

	/**
	 * COしたターン数
	 *してなければ最初に結果を言ったターン
	 *
	 * @return
	 */
	public int getTurn() {
		return turn_;
	}

	/**
	 * 占い(霊能)結果を追加
	 *
	 * @param agentId
	 *            エージェントのIndex
	 * @param isBlack
	 *            黒ならtrue
	 */
	public void add(int day, int targetIdx, boolean isBlack) {
		getDayList().add(day);
		getKnownList().add(targetIdx);
		getIsBlackList().add(isBlack);
	}

	/**
	 * n 日目の結果！
	 *
	 * @param day
	 * @return エージェントindex
	 */
	public abstract int getAgent(int day);

	/**
	 * n 日目の結果！
	 *
	 * @param day
	 * @return 黒なら 1, 白なら 0, それ以外は -1
	 */
	public abstract HashMap<Integer, Boolean> getResult(int day);

	/**
	 * 結果を言った回数
	 *
	 * @return
	 */
	public int getResultNum() {
		return getKnownList().size();
	}

	/**
	 * 黒だししたエージェントリストを取得
	 *
	 * @return 黒だししたエージェントリスト
	 */
	public ArrayList<Integer> getBlackList() {
		ArrayList<Integer> blackList = new ArrayList<Integer>();
		int size = getIsBlackList().size();
		for (int i = 0; i < size; i++) {
			if (getIsBlackList().get(i)) {
				blackList.add(getKnownList().get(i));
			}
		}

		return blackList;
	}

	/**
	 * 白だししたエージェントリストを取得
	 *
	 * @return 白だししたエージェントリスト
	 */
	public ArrayList<Integer> getWhiteList() {
		ArrayList<Integer> whiteList = new ArrayList<Integer>();
		int size = getIsBlackList().size();
		for (int i = 0; i < size; i++) {
			if (!getIsBlackList().get(i)) {
				whiteList.add(getKnownList().get(i));
			}
		}

		return whiteList;
	}

	/**
	 * すべての日の分結果を言ったかどうか
	 * 注) その日の最初に呼ぶべきではない！結果を言うのが２ターン目だったり，
	 * 遅れてCOして，まとめて結果を言っている最中かもしれないので，
	 * 使うならtalkが全て終わってから！！
	 * @param day 現在の日にち
	 * @return
	 */
	public abstract boolean isFullResult(int day);

	public ArrayList<Integer> getDayList() {
		return dayList_;
	}

	public void setDayList(ArrayList<Integer> dayList) {
		dayList_ = dayList;
	}

	public ArrayList<Integer> getKnownList() {
		return knownList_;
	}

	public void setKnownList(ArrayList<Integer> knownList) {
		knownList_ = knownList;
	}

	public ArrayList<Boolean> getIsBlackList() {
		return isBlackList_;
	}

	public void setIsBlackList(ArrayList<Boolean> isBlackList) {
		isBlackList_ = isBlackList;
	}

	public boolean isAlive() {
		return isAlive_;
	}

	public void setAlive(boolean isAlive) {
		isAlive_ = isAlive;
	}

	public boolean isAfterSlide() {
		return isAfterSlide_;
	}

	public void setAfterSlide(boolean isAfterSlide) {
		isAfterSlide_ = isAfterSlide;
	}

}
