package com.gmail.sg06.greenrevolution.data;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Random;

import org.aiwolf.client.lib.Topic;
import org.aiwolf.client.lib.Utterance;
import org.aiwolf.common.data.Agent;
import org.aiwolf.common.data.Role;
import org.aiwolf.common.data.Species;
import org.aiwolf.common.data.Talk;
import org.aiwolf.common.data.Vote;
import org.aiwolf.common.net.GameInfo;

import com.gmail.sg06.greenrevolution.jingai_table.LGJingaiTable;

/**
 * 必要そうなゲームの情報をまとめたクラス！ エージェント情報は基本的にエージェントIndexで扱う！
 *
 * @author hasegawa
 *
 */
public class MyData {

	private final int AGENT_NUM = 15;

	private Random r_; // 乱数

	private LinkedHashMap<Integer, COData> seerCODataMap_; // 占COしたエージェントの情報

	private LinkedHashMap<Integer, COData> mediumCODataMap_; // 霊COしたエージェントの情報

	private LinkedHashMap<Integer, BGCOData> bgCODataMap_; // 狩COしたエージェントの情報

	private LinkedHashMap<Integer, Integer> villCODayMap_;// 村人COしたエージェントのCO日情報

	private LinkedHashMap<Integer, Integer> wolfCODayMap_;// 人狼COしたエージェントのCO日情報

	private LinkedHashMap<Integer, Integer> possCODayMap_;// 狂人COしたエージェントのCO日情報

	private HashMap<Integer, PlayerData> playerDataMap_; // 各プレイヤーの行動マップ

	private HashMap<Integer, Integer> talkNumMap_; // talk回数(skip, over除く)

	private HashMap<Integer, Integer> talkAllNumMap_; // talk回数

	private HashMap<Integer, Integer> voteMap_; // vote宣言したプレイヤとされたプレイヤ

	private HashMap<Integer, Integer> voteNum_; // vote宣言された回数

	private HashMap<Integer, ArrayList<Integer>> resultMap_;

	private ArrayList<Integer> aliveAgentList_; // 生存しているエージェントのリスト

	private ArrayList<Integer> attackedAgentList_; // 襲撃されたエージェントのリスト

	private ArrayList<Integer> executedAgentList_; // 吊られたエージェントのリスト

	// private AbstractJingaiTable wolfSideTable_; // 人外テーブル
	private LGJingaiTable wolfSideTable_; // 人外テーブル

	private ArrayList<int[]> voteTable_; // 投票テーブル

	private int today_; // 日数

	private int latestTalkId_; // 取得した最新の Talk 番号

	private boolean isCall_; // その日に結果が言われたもしくは CO されるたどうか

	private boolean isOneTimeCall_; // その日に結果が言われたもしくは CO されるかどうか(1度きり)

	/**
	 * 引数なしコンストラクタ
	 */
	public MyData() {
		init();
		// wolfSideTable_ = new DefaultJingaiTable();
		wolfSideTable_ = new LGJingaiTable();
	}

	/**
	 * 初期化関数． データを全てクリアする． 100試合，インスタンス破棄されないため， もしそのまま使うならこれを使うかnewし直すかする．
	 */
	public void init() {
		r_ = new Random(0);
		seerCODataMap_ = new LinkedHashMap<Integer, COData>();
		mediumCODataMap_ = new LinkedHashMap<Integer, COData>();
		bgCODataMap_ = new LinkedHashMap<Integer, BGCOData>();
		villCODayMap_ = new LinkedHashMap<Integer, Integer>();
		wolfCODayMap_ = new LinkedHashMap<Integer, Integer>();
		possCODayMap_ = new LinkedHashMap<Integer, Integer>();
		playerDataMap_ = new HashMap<Integer, PlayerData>();
		talkNumMap_ = new HashMap<Integer, Integer>();
		talkAllNumMap_ = new HashMap<Integer, Integer>();
		voteMap_ = new HashMap<Integer, Integer>();
		voteNum_ = new HashMap<Integer, Integer>();
		aliveAgentList_ = new ArrayList<Integer>();
		attackedAgentList_ = new ArrayList<Integer>();
		executedAgentList_ = new ArrayList<Integer>();
		voteTable_ = new ArrayList<int[]>();
		resultMap_ = new HashMap<Integer, ArrayList<Integer>>();
		today_ = 0;
		latestTalkId_ = 0;
		isCall_ = false;
		isOneTimeCall_ = false;
		;


		for (int i = 0; i < 15; i++) {
			aliveAgentList_.add(i + 1);
		}
	}

	/**
	 *
	 * @param gameInfo
	 * @TODO 中身の実装．前回UPDateしてから現在までの情報を更新する！
	 */
	public void update(GameInfo gameInfo) {
		if (gameInfo.getTalkList() != null) {
			for (int i = latestTalkId_; i < gameInfo.getTalkList().size(); i++) {
				Talk talk = gameInfo.getTalkList().get(i);
				Utterance uttr = new Utterance(talk.getContent());
				int agentIdx = talk.getAgent().getAgentIdx();
				// System.out.println(talkAllNumMap_.size() + " " + agentIdx);
				int turn = talkAllNumMap_.get(agentIdx);// ターンは発言回数で定義
				boolean isBlack = false;
				PlayerData p_data;

				// coした人いたら newSeerData(int agentIdx, int day, int
				// turn)やnewMediumData(...)やnewBGDataをよぶ
				if (uttr.getTopic() == Topic.COMINGOUT) {
					if (uttr.getRole() == Role.SEER) {
						newSeerData(agentIdx, today_, turn);
						p_data = playerDataMap_.get(Integer.valueOf(agentIdx));
						p_data.addCOData(uttr.getRole(), today_, turn);
						resultMap_.put(agentIdx, new ArrayList<Integer>());
					} else if (uttr.getRole() == Role.MEDIUM) {
						newMediumData(agentIdx, today_, turn);
						p_data = playerDataMap_.get(agentIdx);
						p_data.addCOData(Role.MEDIUM, today_, turn);

					} else if (uttr.getRole() == Role.BODYGUARD) {
						newBGData(agentIdx, today_, turn);
						p_data = playerDataMap_.get(agentIdx);
						p_data.addCOData(Role.BODYGUARD, today_, turn);

					} else if (uttr.getRole() == Role.VILLAGER) {
						villCODayMap_.put(uttr.getTarget().getAgentIdx(),
								today_);
					} else if (uttr.getRole() == Role.WEREWOLF) {
						wolfCODayMap_.put(uttr.getTarget().getAgentIdx(),
								today_);
					} else if (uttr.getRole() == Role.POSSESSED) {
						possCODayMap_.put(uttr.getTarget().getAgentIdx(),
								today_);
					}

					isCall_ = true;
					isOneTimeCall_ = true;

					// 結果を言った人がいたらaddSeerData(int agentIdx, int targetInx,
					// boolean isBlack,
					// int day, int turn)やaddMediumData(...)やaddBGDataを呼ぶ
				} else if (uttr.getTopic() == Topic.DIVINED) {
					int targetIdx = uttr.getTarget().getAgentIdx();
					ArrayList<Integer> resultList;
					if (!resultMap_.containsKey(agentIdx)) {
						resultList = new ArrayList<Integer>();
						resultMap_.put(agentIdx, resultList);
					} else {
						resultList = resultMap_.get(agentIdx);
					}
					if (uttr.getResult() == Species.WEREWOLF) {
						isBlack = true;
					}
					if (isBlack) {
						resultList.add(1);
					} else {
						resultList.add(0);
					}
					addSeerData(agentIdx, targetIdx, isBlack, today_, turn);
					isCall_ = true;
					isOneTimeCall_ = true;
				} else if (uttr.getTopic() == Topic.INQUESTED) {
					int targetId = uttr.getTarget().getAgentIdx();
					if (uttr.getResult() == Species.WEREWOLF) {
						isBlack = true;
					}
					addMediumData(agentIdx, targetId, isBlack, today_, turn);
					isCall_ = true;
					isOneTimeCall_ = true;
				} else if (uttr.getTopic() == Topic.GUARDED) {
					int targetIdx = uttr.getTarget().getAgentIdx();
					this.addBGData(agentIdx, targetIdx, today_, turn);
					isCall_ = true;
					isOneTimeCall_ = true;
				}

				p_data = playerDataMap_.get(agentIdx);
				p_data.checkSayHello(uttr);

				// 人外テーブルを規格化する
				wolfSideTable_.normalizeWolfSideTable();

				// vote発言をしたら,宣言された数をカウント
				if (uttr.getTopic() == Topic.VOTE) {
					int newTargetIdx = uttr.getTarget().getAgentIdx();
					int oldTargetIdx = voteMap_.get(agentIdx);

					// vote発言によりvoteNum_を＋1する
					if (oldTargetIdx != newTargetIdx) {
						// 投票先を変えたら前のtargetのvoteNum_を減らす
						if (oldTargetIdx != -1) {
							int value = voteNum_.get(oldTargetIdx);
							value--;
							voteNum_.put(oldTargetIdx, value);
						}
						int value = voteNum_.get(newTargetIdx);
						value++;
						voteNum_.put(newTargetIdx, value);
						voteMap_.put(agentIdx, newTargetIdx);
					}
				}

				// 発言した内容がskip,vote以外ならtalk数更新
				if (uttr.getTopic() != Topic.SKIP
						&& uttr.getTopic() != Topic.OVER) {
					int num = talkNumMap_.get(agentIdx);
					num++;
					talkNumMap_.put(agentIdx, num);
				}
				int num = talkAllNumMap_.get(agentIdx);
				num++;
				talkAllNumMap_.put(agentIdx, num);
			}
			latestTalkId_ = gameInfo.getTalkList().size();
		}
	}

	/**
	 *
	 * @param gameInfo
	 * @TODO 中身の実装 updateとまとめてもらってもOKです．
	 */
	public void dayStart(GameInfo gameInfo) {

		today_++;// updateとまとめるならココ注意！！
		isCall_ = false;
		isOneTimeCall_ = false;
		PlayerData p_data;

		List<Agent> list = gameInfo.getAgentList();

		// 生存者のリストを更新する.
		aliveAgentList_.clear();
		for (Agent agent : gameInfo.getAliveAgentList()) {
			aliveAgentList_.add(agent.getAgentIdx());
		}

		// addExecutedAgentを呼ぶ．初日は -1 を渡す
		int executedAgentIdx = -1;
		if (gameInfo.getExecutedAgent() != null) {
			executedAgentIdx = gameInfo.getExecutedAgent().getAgentIdx();
			addExecutedAgentList(executedAgentIdx);
		}

		// addAttackedAgentを呼ぶ．GJなら-1を渡してもよい
		// setAliveをよぶ．GJなら-1を渡すとよい
		// どちらも GJ なのか吊りと噛みが同一だったのかは判断していない
		int attackedAgentIdx = -1;
		if (gameInfo.getAttackedAgent() != null) {
			attackedAgentIdx = gameInfo.getAttackedAgent().getAgentIdx();
		}
		addAttackedAgentList(attackedAgentIdx);
		setAlive(executedAgentIdx, attackedAgentIdx);

		// voteTable を更新する. 初日や死亡によって投票できていない場合は -1 とする.
		// ここ以外で投票テーブルを更新することはないはずなので, メソッドにしない.
		int[] dayVoteList = new int[15];
		for (int i = 0; i < dayVoteList.length; i++) {
			dayVoteList[i] = -1;
		}
		voteTable_.add(dayVoteList);
		for (Vote vote : gameInfo.getVoteList()) {
			voteTable_.get(vote.getDay())[vote.getAgent().getAgentIdx() - 1] = vote
					.getTarget().getAgentIdx();
		}

		// votepatternのチェック
		for (Vote vote : gameInfo.getVoteList()) {
			Integer agentIdx = vote.getAgent().getAgentIdx();
			Integer voteIdx = vote.getTarget().getAgentIdx();
			Integer talkedIdx = voteMap_.get(agentIdx);
			Integer votepattern;

			if (voteIdx == talkedIdx) {
				votepattern = 0;
			} // 宣言==投票先
			else {
				if (talkedIdx != -1) {
					votepattern = 1;
				} // 宣言!=投票先
				else {
					votepattern = 2;
				} // 宣言なし
			}

			p_data = playerDataMap_.get(agentIdx);
			if (today_ > 1 && getCurrentSeerCOAgentList().contains(agentIdx)) {
				p_data.addDayData(votepattern, resultMap_.get(agentIdx));
			} else if (today_ > 0) {
				p_data.addDayData(votepattern);
			}
		}

		// talknum_,voteMap_リセット
		for (Agent agent : list) {
			talkNumMap_.put(agent.getAgentIdx(), 0);
			talkAllNumMap_.put(agent.getAgentIdx(), 0);
			voteMap_.put(agent.getAgentIdx(), -1);
			voteNum_.put(agent.getAgentIdx(), 0);
		}
	}

	/**
	 *
	 * @param gameInfo
	 */
	public void gameStart(GameInfo gameInfo) {
		List<Agent> list = gameInfo.getAgentList();
		for (Agent agent : list) {
			PlayerData pd = new PlayerData(agent.getAgentIdx(),
					agent.toString());
			playerDataMap_.put(agent.getAgentIdx(), pd);
			talkNumMap_.put(agent.getAgentIdx(), 0);
			talkAllNumMap_.put(agent.getAgentIdx(), 0);
			voteMap_.put(agent.getAgentIdx(), -1);
			voteNum_.put(agent.getAgentIdx(), 0);

			// 人外テーブル更新用の倍率テーブルを初期化する
			wolfSideTable_.initTable(true);
			wolfSideTable_.agentMemory_.clear();
			wolfSideTable_.roleMemory_.clear();
			wolfSideTable_.infoMemory_.clear();


		}
	}

	/**
	 * talk終了(vote時)に呼ぶ
	 */
	public void talkEnd() {
		PlayerData p_data;
		for (Integer i : playerDataMap_.keySet()) {
			p_data = playerDataMap_.get(i);
			p_data.addTalkData(talkNumMap_.get(i));
		}
		if (getToday() == 0) {

		}
	}

	/**
	 * 吊られた，噛まれた人を更新するときによぶ！
	 *
	 * @param deadIdx
	 */
	private void setAlive(int executedIdx, int attackedIdx) {
		for (java.util.Map.Entry<Integer, COData> entry : seerCODataMap_
				.entrySet()) {
			int key = entry.getKey();
			if (key == executedIdx || key == attackedIdx) {
				entry.getValue().setAlive(false);
			}
		}
		for (java.util.Map.Entry<Integer, COData> entry : mediumCODataMap_
				.entrySet()) {
			int key = entry.getKey();
			if (key == executedIdx || key == attackedIdx) {
				entry.getValue().setAlive(false);
			}
		}
		for (java.util.Map.Entry<Integer, BGCOData> entry : bgCODataMap_
				.entrySet()) {
			int key = entry.getKey();
			if (key == executedIdx || key == attackedIdx) {
				entry.getValue().setAlive(false);
			}
		}
	}

	/**
	 * COした人がいたら新しいデータインスタンスを作って追加
	 *
	 * @param agentIdx
	 *            COした人の agent index
	 * @param day
	 *            COした日にち
	 * @param turn
	 *            COしたターン数
	 */
	private void newSeerData(int agentIdx, int day, int turn) {
		if (!seerCODataMap_.containsKey(agentIdx)) {// 何度もCOと言う人いるので
			COData data = new SeerCOData(day, turn, true);
			seerCODataMap_.put(agentIdx, data);
			if (mediumCODataMap_.containsKey(agentIdx)) {
				data.setAfterSlide(true);
				mediumCODataMap_.get(agentIdx).setAlive(false);
			}
			if (bgCODataMap_.containsKey(agentIdx)) {
				data.setAfterSlide(true);
				bgCODataMap_.get(agentIdx).setAlive(false);
			}

		}
	}

	/**
	 * COした人がいたら新しいデータインスタンスを作って追加
	 *
	 * @param agentIdx
	 *            COした人の agent index
	 * @param day
	 *            COした日にち
	 * @param turn
	 *            COしたターン数
	 */
	private void newMediumData(int agentIdx, int day, int turn) {
		if (!mediumCODataMap_.containsKey(agentIdx)) {// 何度もCOと言う人いるので
			COData data = new MediumCOData(day, turn, true);
			mediumCODataMap_.put(agentIdx, data);
			if (seerCODataMap_.containsKey(agentIdx)) {
				data.setAfterSlide(true);
				seerCODataMap_.get(agentIdx).setAlive(false);
			}
			if (bgCODataMap_.containsKey(agentIdx)) {
				data.setAfterSlide(true);
				bgCODataMap_.get(agentIdx).setAlive(false);
			}
		}
	}

	/**
	 * COした人がいたら新しいデータインスタンスを作って追加
	 *
	 * @param agentIdx
	 *            COした人の agent index
	 * @param day
	 *            COした日にち
	 * @param turn
	 *            COしたターン数
	 */
	private void newBGData(int agentIdx, int day, int turn) {
		if (!bgCODataMap_.containsKey(agentIdx)) {// 何度もCOと言う人いるので
			BGCOData data = new BGCOData(day, turn, true);
			bgCODataMap_.put(agentIdx, data);
			if (mediumCODataMap_.containsKey(agentIdx)) {
				data.setAfterSlide(true);
				mediumCODataMap_.get(agentIdx).setAlive(false);
			}
			if (seerCODataMap_.containsKey(agentIdx)) {
				data.setAfterSlide(true);
				seerCODataMap_.get(agentIdx).setAlive(false);
			}
		}
	}

	/**
	 * 結果を言った人がいたらデータを追加する
	 *
	 * @param agentIdx
	 *            結果を言った人のindex
	 * @param targetInx
	 *            占い先のindex
	 * @param isBlack
	 *            黒かどうか
	 * @param day
	 *            言った日にち
	 * @param turn
	 *            言ったターン
	 */
	private void addSeerData(int agentIdx, int targetIdx, boolean isBlack,
			int day, int turn) {

		COData data;
		if (!seerCODataMap_.containsKey(agentIdx)) {// 何度もCOと言う人いるので
			data = new SeerCOData(day, turn, false);
			seerCODataMap_.put(agentIdx, data);
			if (mediumCODataMap_.containsKey(agentIdx)) {
				data.setAfterSlide(true);
				mediumCODataMap_.get(agentIdx).setAlive(false);
			}
			if (bgCODataMap_.containsKey(agentIdx)) {
				data.setAfterSlide(true);
				bgCODataMap_.get(agentIdx).setAlive(false);
			}
		} else {
			data = seerCODataMap_.get(agentIdx);
		}
		data.add(day, targetIdx, isBlack);
	}

	/**
	 * 結果を言った人がいたらデータを追加する
	 *
	 * @param agentIdx
	 *            結果を言った人のindex
	 * @param targetInx
	 *            霊能先のindex
	 * @param isBlack
	 *            黒かどうか
	 * @param day
	 *            言った日にち
	 * @param turn
	 *            言ったターン
	 */
	private void addMediumData(int agentIdx, int targetIdx, boolean isBlack,
			int day, int turn) {
		COData data;
		if (!mediumCODataMap_.containsKey(agentIdx)) {// 何度もCOと言う人いるので
			data = new MediumCOData(day, turn, false);
			mediumCODataMap_.put(agentIdx, data);
			if (seerCODataMap_.containsKey(agentIdx)) {
				data.setAfterSlide(true);
				seerCODataMap_.get(agentIdx).setAlive(false);
			}
			if (bgCODataMap_.containsKey(agentIdx)) {
				data.setAfterSlide(true);
				bgCODataMap_.get(agentIdx).setAlive(false);
			}
		} else {
			data = mediumCODataMap_.get(agentIdx);
		}
		data.add(day, targetIdx, isBlack);
	}

	/**
	 * 結果を言った人がいたらデータを追加する
	 *
	 * @param agentIdx
	 *            結果を言った人のindex
	 * @param targetInx
	 *            守り先のindex
	 * @param isBlack
	 *            黒かどうか
	 * @param day
	 *            言った日にち
	 * @param turn
	 *            言ったターン
	 */
	private void addBGData(int agentIdx, int targetIdx, int day, int turn) {
		BGCOData data;
		if (!bgCODataMap_.containsKey(agentIdx)) {// 何度もCOと言う人いるので
			data = new BGCOData(day, turn, false);
			bgCODataMap_.put(agentIdx, data);
			if (mediumCODataMap_.containsKey(agentIdx)) {
				data.setAfterSlide(true);
				mediumCODataMap_.get(agentIdx).setAlive(false);
			}
			if (seerCODataMap_.containsKey(agentIdx)) {
				data.setAfterSlide(true);
				seerCODataMap_.get(agentIdx).setAlive(false);
			}
		} else {
			data = bgCODataMap_.get(agentIdx);
		}
		data.add(targetIdx);
	}

	/**
	 * 占いCOしたエージェントリスト（結果のみ含む）
	 *
	 * @return
	 */
	public ArrayList<Integer> getSeerCOAgentList() {
		ArrayList<Integer> reList = new ArrayList<Integer>();
		for (java.util.Map.Entry<Integer, COData> entry : seerCODataMap_
				.entrySet()) {
			reList.add(entry.getKey());
		}
		return reList;
	}

	/**
	 * 霊能COしたエージェントリスト（結果のみ含む）
	 *
	 * @return
	 */
	public ArrayList<Integer> getMediumCOAgentList() {
		ArrayList<Integer> reList = new ArrayList<Integer>();
		for (java.util.Map.Entry<Integer, COData> entry : mediumCODataMap_
				.entrySet()) {
			reList.add(entry.getKey());
		}
		return reList;
	}

	/**
	 * 狩人COしたエージェントリスト（結果のみ含む）
	 *
	 * @return
	 */
	public ArrayList<Integer> getBGCOAgentList() {
		ArrayList<Integer> reList = new ArrayList<Integer>();
		for (java.util.Map.Entry<Integer, BGCOData> entry : bgCODataMap_
				.entrySet()) {
			reList.add(entry.getKey());
		}
		return reList;
	}

	/**
	 * 生きていてスライドも考慮した現在の占い候補リスト（結果のみ含む）
	 *
	 * @TODO スライドCO判定
	 * @return
	 */
	public ArrayList<Integer> getCurrentSeerCOAgentList() {
		ArrayList<Integer> reList = new ArrayList<Integer>();
		for (java.util.Map.Entry<Integer, COData> entry : seerCODataMap_
				.entrySet()) {
			if (entry.getValue().isAlive()) {
				reList.add(entry.getKey());
			}
		}
		return reList;
	}

	/**
	 * 生きていてスライドも考慮した現在の霊能候補リスト（結果のみ含む）
	 *
	 * @TODO スライドCO判定
	 * @return
	 */
	public ArrayList<Integer> getCurrentMediumCOAgentList() {
		ArrayList<Integer> reList = new ArrayList<Integer>();
		for (java.util.Map.Entry<Integer, COData> entry : mediumCODataMap_
				.entrySet()) {
			if (entry.getValue().isAlive()) {
				reList.add(entry.getKey());
			}
		}
		return reList;
	}

	/**
	 * 生きていてスライドも考慮した現在の狩人候補リスト（結果のみ含む）
	 *
	 * @TODO スライドCO判定
	 * @return
	 */
	public ArrayList<Integer> getCurrentBGCOAgentList() {
		ArrayList<Integer> reList = new ArrayList<Integer>();
		for (java.util.Map.Entry<Integer, BGCOData> entry : bgCODataMap_
				.entrySet()) {
			if (entry.getValue().isAlive()) {
				reList.add(entry.getKey());
			}
		}
		return reList;
	}

	/**
	 * 生きていてスライドも考慮した当日の占い候補リスト（結果のみ含む）
	 *
	 * @TODO スライドCO判定
	 * @return
	 */
	public ArrayList<Integer> getTodaySeerCOAgentList() {
		ArrayList<Integer> reList = new ArrayList<Integer>();
		for (java.util.Map.Entry<Integer, COData> entry : seerCODataMap_
				.entrySet()) {
			if (entry.getValue().isAlive()
					&& (entry.getValue().getDay() == today_)) {
				reList.add(entry.getKey());
			}
		}
		return reList;
	}

	/**
	 * 生きていてスライドも考慮した当日の霊能候補リスト（結果のみ含む）
	 *
	 * @TODO スライドCO判定
	 * @return
	 */
	public ArrayList<Integer> getTodayMediumCOAgentList() {
		ArrayList<Integer> reList = new ArrayList<Integer>();
		for (java.util.Map.Entry<Integer, COData> entry : mediumCODataMap_
				.entrySet()) {
			if (entry.getValue().isAlive()
					&& (entry.getValue().getDay() == today_)) {
				reList.add(entry.getKey());
			}
		}
		return reList;
	}

	/**
	 * 生きていてスライドも考慮した当日の狩人候補リスト（結果のみ含む）
	 *
	 * @TODO スライドCO判定
	 * @return
	 */
	public ArrayList<Integer> getTodayBGCOAgentList() {
		ArrayList<Integer> reList = new ArrayList<Integer>();
		for (java.util.Map.Entry<Integer, BGCOData> entry : bgCODataMap_
				.entrySet()) {
			if (entry.getValue().isAlive()
					&& (entry.getValue().getDay() == today_)) {
				reList.add(entry.getKey());
			}
		}
		return reList;
	}

	/**
	 * 噛まれたエージェントリスト
	 *
	 * @return
	 */
	public ArrayList<Integer> getAttackedAgentList() {
		return attackedAgentList_;
	}

	/**
	 * 誰が誰に投票したかのテーブル (投票テーブル)
	 *
	 * @param day
	 *            欲しい投票テーブルの日数
	 * @return
	 */
	public int[] getVoteTable(int day) {
		return voteTable_.get(day);
	}

	/**
	 * 指定した人の結果を取ってくる COしていなければnullが返ってくる．
	 *
	 * @return
	 */
	public ArrayList<Integer> getSeerResultsList(int index) {
		if (seerCODataMap_.containsKey(index)) {
			return seerCODataMap_.get(index).getKnownList();
		} else {
			return null;
		}
	}

	/**
	 * 指定した人の結果を取ってくる COしていなければnullが返ってくる．
	 *
	 * @return
	 */
	public ArrayList<Integer> getMediumResultsList(int index) {
		if (mediumCODataMap_.containsKey(index)) {
			return mediumCODataMap_.get(index).getKnownList();
		} else {
			return null;
		}
	}

	/**
	 * SEER COの全データのgetter
	 *
	 * @return
	 */
	public LinkedHashMap<Integer, COData> getSeerCODataMap() {
		return seerCODataMap_;
	}

	public void setSeerCODataMap(LinkedHashMap<Integer, COData> seerCODataMap) {
		seerCODataMap_ = seerCODataMap;
	}

	/**
	 * MEDIUM COの全データのgetter
	 *
	 * @return
	 */
	public LinkedHashMap<Integer, COData> getMediumCODataMap() {
		return mediumCODataMap_;
	}

	public void setMediumCODataMap(
			LinkedHashMap<Integer, COData> mediumCODataMap) {
		mediumCODataMap_ = mediumCODataMap;
	}

	public ArrayList<Integer> getExecutedAgentList() {
		return executedAgentList_;
	}

	public ArrayList<Integer> getAliveAgentList() {
		return aliveAgentList_;
	}

	public void addExecutedAgentList(Integer votedAgent) {
		executedAgentList_.add(votedAgent);
	}

	public void addAttackedAgentList(Integer attackedAgent) {
		attackedAgentList_.add(attackedAgent);
	}

	/**
	 * BODYGUARD COの全データのgetter
	 *
	 * @return
	 */
	public LinkedHashMap<Integer, BGCOData> getBgCODataMap() {
		return bgCODataMap_;
	}

	public void setBgCODataMap(LinkedHashMap<Integer, BGCOData> bgCODataMap) {
		bgCODataMap_ = bgCODataMap;
	}

	/**
	 * 人外テーブルを更新する. upType = 0 ならば dayStart or 会話フェイズに実行する. (なるべく dayStart に実行)
	 * upType = 1 ならば 基本的には vote の直前に実行する. (会話フェイズで実行するかは player 依存.)
	 *
	 * @param upType
	 *            更新する種類( 0: 襲撃・投票, 1: CO 関連・会話関連)
	 */
	public void tableUpdate(int upType) {
		// 倍率テーブルの初期化
		// wolfSideTable_.initRateTable(upType);
		wolfSideTable_.initTable(false);

		// 倍率テーブルの更新 (襲撃・投票)
		if (upType == 0) {
			int yesterday = today_ - 1;
			// 襲撃
			int attackedAgentIdx = attackedAgentList_.get(yesterday);
			if (attackedAgentIdx != -1) {
				wolfSideTable_.calcAttackInfo(attackedAgentIdx);
			}
			// 投票
			int[] votes = getVoteTable(yesterday);
			for (int i = 0; i < votes.length; i++) {
				if (votes[i] > 0) {
					// wolfSideTable_.calcVoteInfo(i + 1, votes[i]);
					wolfSideTable_.calcVoteInfo((i + 1), votes[i], yesterday);
				}
			}

			// 倍率テーブルの更新 (CO 関連)
		} else if (upType == 1) {
			// CO関連
			wolfSideTable_.calcSeerCOInfo(getSeerCOAgentList());
			wolfSideTable_.calcMediumCOInfo(getMediumCOAgentList());
			wolfSideTable_.calcBGCOInfo(getBGCOAgentList());
			ArrayList<Integer> notCOAgentList = new ArrayList<Integer>();
			for (int idx : aliveAgentList_) {
				if (!getCurrentSeerCOAgentList().contains(idx)
						&& !getCurrentMediumCOAgentList().contains(idx)
						&& !getCurrentBGCOAgentList().contains(idx)) {
					notCOAgentList.add(idx);
				}
			}
			wolfSideTable_.calcNotCOInfo(notCOAgentList);
			// 会話関連
			for (java.util.Map.Entry<Integer, COData> entry : seerCODataMap_
					.entrySet()) {
				int agentIdx = entry.getKey();
				HashMap<Integer, Boolean> result = entry.getValue().getResult(
						today_);
				if (result != null) {
					for (int targetIdx : result.keySet()) {
						boolean isBlack = result.get(targetIdx);
						if (isBlack) {
							wolfSideTable_.calcDivineInfo(agentIdx, targetIdx,
									true);
						} else {
							wolfSideTable_.calcDivineInfo(agentIdx, targetIdx,
									false);
						}
					}
				}
			}
			for (java.util.Map.Entry<Integer, COData> entry : mediumCODataMap_
					.entrySet()) {
				int agentIdx = entry.getKey();
				HashMap<Integer, Boolean> result = entry.getValue().getResult(
						today_);
				if (result != null) {
					for (int targetIdx : result.keySet()) {
						boolean isBlack = result.get(targetIdx);
						if (isBlack) {
							wolfSideTable_.calcInquestInfo(agentIdx, targetIdx,
									true);
						} else {
							wolfSideTable_.calcInquestInfo(agentIdx, targetIdx,
									false);
						}
					}
				}
			}
			for (int seerIdx : seerCODataMap_.keySet()) {
				SeerCOData seerCO = (SeerCOData) seerCODataMap_.get(Integer
						.valueOf(seerIdx));
				for (int mediumIdx : mediumCODataMap_.keySet()) {
					MediumCOData mediumCO = (MediumCOData) mediumCODataMap_
							.get(Integer.valueOf(mediumIdx));
					wolfSideTable_.calcPatternInfo(seerIdx, seerCO, mediumIdx,
							mediumCO);
				}
			}

			// その他 (エラー)
		} else {
			System.out.println("`upType' is wrong number! (upType is 0 or 1)");
		}

		// 人外テーブルの更新
		wolfSideTable_.update();
	}

	/**
	 * 試合結果から人外テーブルの倍率を学習する. finish() 以外では使用しない.
	 *
	 * @param gameInfo
	 */
	public void learningTableRate(GameInfo gameInfo) {
		wolfSideTable_.learningChangeRate(gameInfo);
		// for ( int i = 0; i < wolfSideTable_.getRates().length; i++ ){
		// System.out.print(String.format("%.5f", wolfSideTable_.getRate(i)) +
		// " , ");
		// }
		// System.out.println();
	}

	/**
	 * もっとも狼らしいエージェントを返す．
	 *
	 * @return
	 */
	public int getMaxLikelyWolf() {
		double maxRate = Double.NEGATIVE_INFINITY;
		ArrayList<Integer> maxIdxList = new ArrayList<Integer>();
		for (int idx : aliveAgentList_) {
			double wolfRate = wolfSideTable_.getWolfRate(idx);
			// System.out.println("idx:" + idx + "  " + wolfRate + "(" +
			// maxIdxList.size() + ")");
			if (maxRate < wolfRate) {
				maxRate = wolfRate;
				maxIdxList.clear();
				maxIdxList.add(idx);
			} else if (maxRate == wolfRate) {
				maxIdxList.add(idx);
			}
		}
		if (maxIdxList.size() < 1) {
			return -1;
		} else if (maxIdxList.size() == 1) {
			return maxIdxList.get(0);
		} else {
			return maxIdxList.get(r_.nextInt(maxIdxList.size()));
		}
	}

	/**
	 * もっとも人狼らしくないエージェントを返す
	 *
	 * @return
	 */
	public int getMinLikelyWolf() {
		double minRate = Double.POSITIVE_INFINITY;
		ArrayList<Integer> minIdxList = new ArrayList<Integer>();
		for (int idx : aliveAgentList_) {
			double nWolfRate = wolfSideTable_.getWolfRate(idx);
			if (minRate > nWolfRate) {
				minRate = nWolfRate;
				minIdxList.clear();
				minIdxList.add(idx);
			} else if (minRate == nWolfRate) {
				minIdxList.add(idx);
			}
		}
		if (minIdxList.size() < 1) {
			return -1;
		} else if (minIdxList.size() == 1) {
			return minIdxList.get(0);
		} else {
			return minIdxList.get(r_.nextInt(minIdxList.size()));
		}
	}

	/**
	 * 候補の中からもっとも狼らしくないエージェントを返す.
	 *
	 * @param targetIdxList
	 * @return
	 */
	public int getMinLikelyWolf(ArrayList<Integer> targetIdxList) {
		double minRate = 5460;
		ArrayList<Integer> minIdxList = new ArrayList<Integer>();
		for (int idx : targetIdxList) {
			if (isAlive(idx)) {
				double wolfRate = wolfSideTable_.getWolfRate(idx);
				if (minRate > wolfRate) {
					minRate = wolfRate;
					minIdxList.clear();
					minIdxList.add(idx);
				} else if (minRate == wolfRate) {
					minIdxList.add(idx);
				}
			}
		}
		if (minIdxList.size() < 1) {
			return -1;
		} else if (minIdxList.size() == 1) {
			return minIdxList.get(0);
		} else {
			return minIdxList.get(r_.nextInt(minIdxList.size()));
		}
	}

	/**
	 * 候補の中からもっとも狼らしいエージェントを返す.
	 *
	 * @param targetIdxList
	 * @return
	 */
	public int getMaxLikelyWolf(ArrayList<Integer> targetIdxList) {
		double maxRate = Double.NEGATIVE_INFINITY;
		ArrayList<Integer> maxIdxList = new ArrayList<Integer>();
		for (int idx : targetIdxList) {
			if (isAlive(idx)) {
				double wolfRate = wolfSideTable_.getWolfRate(idx);
				if (maxRate < wolfRate) {
					maxRate = wolfRate;
					maxIdxList.clear();
					maxIdxList.add(idx);
				} else if (maxRate == wolfRate) {
					maxIdxList.add(idx);
				}
			}
		}
		if (maxIdxList.size() < 1) {
			return -1;
		} else if (maxIdxList.size() == 1) {
			return maxIdxList.get(0);
		} else {
			return maxIdxList.get(r_.nextInt(maxIdxList.size()));
		}
	}

	/**
	 * 現在生存しているエージェントそれぞれのもっとも狼らしい確率(?)を返す.
	 *
	 * @return
	 */
	public ArrayList<Double> getLikelyWolfRate() {
		ArrayList<Double> wolfRates = new ArrayList<Double>();
		for (int agentIdx : getAliveAgentList()) {
			wolfRates.add(wolfSideTable_.getWolfRate(agentIdx));
		}

		return wolfRates;
	}

	public HashMap<Integer, Double> getLikelyWolfRateMap() {
		HashMap<Integer, Double> wolfRates = new HashMap<Integer, Double>();
		for (int agentIdx : getAliveAgentList()) {
			wolfRates.put(agentIdx, wolfSideTable_.getWolfRate(agentIdx));
		}

		return wolfRates;
	}

	/**
	 * もっとも村人陣営らしいエージェントを返す．
	 *
	 * @return
	 * @TODO 実装
	 */
	public int getMaxLikelyVill() {
		double maxRate = Double.NEGATIVE_INFINITY;
		ArrayList<Integer> maxIdxList = new ArrayList<Integer>();
		for (int idx : aliveAgentList_) {
			double nWolfRate = wolfSideTable_.getNotWolfRate(idx);
			if (maxRate < nWolfRate) {
				maxRate = nWolfRate;
				maxIdxList.clear();
				maxIdxList.add(idx);
			} else if (maxRate == nWolfRate) {
				maxIdxList.add(idx);
			}
		}
		if (maxIdxList.size() < 1) {
			return -1;
		} else if (maxIdxList.size() == 1) {
			return maxIdxList.get(0);
		} else {
			return maxIdxList.get(r_.nextInt(maxIdxList.size()));
		}
	}

	/**
	 * もっとも村人陣営らしくないエージェントを返す
	 *
	 * @return
	 */
	public int getMinLikelyVill() {
		double minRate = Double.POSITIVE_INFINITY;
		ArrayList<Integer> minIdxList = new ArrayList<Integer>();
		for (int idx : aliveAgentList_) {
			double nWolfRate = wolfSideTable_.getNotWolfRate(idx);
			if (minRate > nWolfRate) {
				minRate = nWolfRate;
				minIdxList.clear();
				minIdxList.add(idx);
			} else if (minRate == nWolfRate) {
				minIdxList.add(idx);
			}
		}
		if (minIdxList.size() < 1) {
			return -1;
		} else if (minIdxList.size() == 1) {
			return minIdxList.get(0);
		} else {
			return minIdxList.get(r_.nextInt(minIdxList.size()));
		}
	}

	/**
	 * 候補の中からもっとも村人陣営らしいエージェントを返す.
	 *
	 * @param targetIdxList
	 * @return
	 */
	public int getMaxLikelyVill(ArrayList<Integer> targetIdxList) {
		double maxRate = Double.NEGATIVE_INFINITY;
		ArrayList<Integer> maxIdxList = new ArrayList<Integer>();
		for (int idx : targetIdxList) {
			if (isAlive(idx)) {
				double nWolfRate = wolfSideTable_.getNotWolfRate(idx);
				if (maxRate < nWolfRate) {
					maxRate = nWolfRate;
					maxIdxList.clear();
					maxIdxList.add(idx);
				} else if (maxRate == nWolfRate) {
					maxIdxList.add(idx);
				}
			}
		}
		if (maxIdxList.size() < 1) {
			return -1;
		} else if (maxIdxList.size() == 1) {
			return maxIdxList.get(0);
		} else {
			return maxIdxList.get(r_.nextInt(maxIdxList.size()));
		}
	}

	/**
	 * 候補の中からもっとも村人陣営らしくないエージェントを返す.
	 *
	 * @param targetIdxList
	 * @return
	 */
	public int getMinLikelyVill(ArrayList<Integer> targetIdxList) {
		double minRate = 5460;
		ArrayList<Integer> minIdxList = new ArrayList<Integer>();
		for (int idx : targetIdxList) {
			if (isAlive(idx)) {
				double wolfRate = wolfSideTable_.getNotWolfRate(idx);
				if (minRate > wolfRate) {
					minRate = wolfRate;
					minIdxList.clear();
					minIdxList.add(idx);
				} else if (minRate == wolfRate) {
					minIdxList.add(idx);
				}
			}
		}
		if (minIdxList.size() < 1) {
			return -1;
		} else if (minIdxList.size() == 1) {
			return minIdxList.get(0);
		} else {
			return minIdxList.get(r_.nextInt(minIdxList.size()));
		}
	}

	/**
	 * 現在生存しているエージェントそれぞれのもっとも村人陣営らしい確率(?)を返す.
	 *
	 * @return
	 */
	public ArrayList<Double> getLikelyVillRate() {
		ArrayList<Double> nWolfRates = new ArrayList<Double>();
		for (int agentIdx : getAliveAgentList()) {
			nWolfRates.add(wolfSideTable_.getNotWolfRate(agentIdx));
		}

		return nWolfRates;
	}

	public HashMap<Integer, Double> getLikelyVillRateMap() {
		HashMap<Integer, Double> nWolfRates = new HashMap<Integer, Double>();
		for (int agentIdx : getAliveAgentList()) {
			nWolfRates.put(agentIdx, wolfSideTable_.getNotWolfRate(agentIdx));
		}

		return nWolfRates;
	}

	public HashMap<Integer, Integer> getLatestVotedNumber() {
		HashMap<Integer, Integer> voteMap = new HashMap<Integer, Integer>();

		for (int i = 0; i < voteTable_.get(today_).length; i++) {
			int targetIdx = voteTable_.get(today_)[i];
			if (voteMap.containsKey(Integer.valueOf(targetIdx))) {
				voteMap.put(Integer.valueOf(targetIdx),
						voteMap.get(Integer.valueOf(targetIdx)) + 1);
			} else {
				voteMap.put(Integer.valueOf(targetIdx), 1);
			}
		}
		return voteMap;
	}

	/**
	 * 現在生存しているエージェントの中で最も投票宣言されたエージェントを返す.
	 *
	 * @return
	 */
	public int getMaxLikelyExecuteAgent() {
		double maxVote = -1;
		ArrayList<Integer> maxIdxList = new ArrayList<Integer>();
		for (int idx : aliveAgentList_) {
			int votenum = voteNum_.get(idx);
			if (maxVote < votenum) {
				maxVote = votenum;
				maxIdxList.clear();
				maxIdxList.add(idx);
			} else if (maxVote == votenum) {
				maxIdxList.add(idx);
			}
		}
		if (maxIdxList.size() < 1) {
			return -1;
		} else if (maxIdxList.size() == 1) {
			return maxIdxList.get(0);
		} else {
			return maxIdxList.get(r_.nextInt(maxIdxList.size()));
		}
	}

	/**
	 * 選択したエージェントの中で最も投票宣言されたエージェントを返す.
	 *
	 * @return
	 */
	public int getMaxLikelyExecuteAgent(ArrayList<Integer> targetIdxList) {
		double maxVote = -1;
		ArrayList<Integer> maxIdxList = new ArrayList<Integer>();
		for (int idx : targetIdxList) {
			int votenum = voteNum_.get(idx);
			if (maxVote < votenum) {
				maxVote = votenum;
				maxIdxList.clear();
				maxIdxList.add(idx);
			} else if (maxVote == votenum) {
				maxIdxList.add(idx);
			}
		}
		if (maxIdxList.size() < 1) {
			return -1;
		} else if (maxIdxList.size() == 1) {
			return maxIdxList.get(0);
		} else {
			return maxIdxList.get(r_.nextInt(maxIdxList.size()));
		}
	}

	/**
	 * 現在生存しているエージェントそれぞれが投票された回数を返す.
	 *
	 * @return
	 */
	public HashMap<Integer, Integer> getLikelyExecuteMap() {
		HashMap<Integer, Integer> votedmap = new HashMap<Integer, Integer>();
		for (int agentIdx : getAliveAgentList()) {
			votedmap.put(agentIdx, voteMap_.get(agentIdx));
		}

		return votedmap;
	}

	/**
	 * このメソッドが呼ばれた日，そのターンまでに結果またはCOのいずれかがされていればtrueを返す
	 *
	 * @return
	 */
	public boolean isCall() {
		return isCall_;
	}

	/**
	 * このメソッドが呼ばれた日，そのターンまでに結果またはCOのいずれかがされていればtrueを返す 一度呼ばれると false に変化する
	 *
	 * @return
	 */
	public boolean isOneTimeCall() {
		boolean answer = isOneTimeCall_;
		isOneTimeCall_ = false;
		return answer;
	}

	/**
	 * このメソッドが呼ばれた時点で, エージェント番号が agentIdx であるエージェントが 生存していれば true, 死んでいれば false
	 * を返す.
	 *
	 * @param agentIdx
	 * @return
	 */
	public boolean isAlive(int agentIdx) {
		if (aliveAgentList_.contains(agentIdx)) {
			return true;
		} else {
			return false;
		}
	}

	public int getToday() {
		return today_;
	}

	public HashMap<Integer, PlayerData> getPlayerDataMap() {
		return playerDataMap_;
	}

	public HashMap<Integer, ArrayList<Integer>> getResultMap() {
		return resultMap_;
	}

	public HashMap<Integer, Integer> getTalkAllNumMap() {
		return talkAllNumMap_;
	}

	public LinkedHashMap<Integer, Integer> getVillCODayMap() {
		return villCODayMap_;
	}

	public LinkedHashMap<Integer, Integer> getWolfCODayMap() {
		return wolfCODayMap_;
	}

	public LinkedHashMap<Integer, Integer> getPossCODayMap() {
		return possCODayMap_;
	}

	public HashMap<Integer, Integer> getVoteMap() {

		return voteMap_;
	}

}
