package com.gmail.sg06.greenrevolution.data;

import java.util.ArrayList;

import org.aiwolf.client.lib.Utterance;
import org.aiwolf.common.data.Role;

import com.gmail.sg06.greenrevolution.weightedplayer.SayHello;

/**
 * 過去の試合データから役職ごとの確率を計算するのに必要な特徴量を保存するクラス
 *
 * @author hasegawa
 *
 */
public class PlayerData {

	private int agentIndex_;

	private String agentName_;

	private int coData_;

	private int coTiming_;

	private ArrayList<Integer> talkNum_;// トークの回数．skip over除く

	private ArrayList<Integer> votePattern_;// voteパターン. 0:
											// 投票宣言先＝投票先，1:ノットイコール，2: 宣言なし

	private ArrayList<Integer> resultBlack_;// 占いCOしていたとして，黒を出したかどうか．（占いCOしてなければ空になる）

	private Role role_;

	private SayHello sh_;

	public PlayerData(int agentIndex, String agentName) {
		agentIndex_ = agentIndex;
		agentName_ = agentName;
		coData_ = 0;
		coTiming_ = -1;
		talkNum_ = new ArrayList<Integer>();
		votePattern_ = new ArrayList<Integer>();
		resultBlack_ = new ArrayList<Integer>();
		sh_ = new SayHello();
	}

	/**
	 * COしたらこれを更新する！
	 *
	 * @param role
	 * @param day
	 * @param turn
	 */
	public void addCOData(Role role, int day, int turn) {
		int timing = (10 * day) + turn;
		if (coTiming_ != -1) {
			coTiming_ = (coTiming_ * 100) + timing;
		} else {
			coTiming_ = timing;
		}
	}

	/**
	 * データ追加（voteの前によぶ）
	 *
	 * @param talkNum
	 */
	public void addTalkData(int talkNum) {
		talkNum_.add(talkNum);
	}

	/**
	 * データ追加（dayStartで呼ぶ（1日目以降））
	 *
	 * @param votePatern
	 */
	public void addDayData(int votePatern) {
		votePattern_.add(votePatern);
	}

	/**
	 * データ追加(daystartで呼ぶ（占いCOしてて結果言いはじめたら))
	 *
	 * @param votePatern
	 * @param isBlack
	 */
	public void addDayData(int votePatern, ArrayList<Integer> isBlack) {
		if (resultBlack_.size() != isBlack.size()) {
			resultBlack_ = isBlack;
		}
		addDayData(votePatern);
	}

	public int getAgentIndex() {
		return agentIndex_;
	}

	public void setAgentIndex(int agentIndex) {
		agentIndex_ = agentIndex;
	}

	public String getAgentName() {
		return agentName_;
	}

	public void setAgentName(String agentName) {
		agentName_ = agentName;
	}

	public int getCoData() {
		return coData_;
	}

	public void setCoData(int coData) {
		coData_ = coData;
	}

	public int getCoTiming() {
		return coTiming_;
	}

	public void setCoTiming(int coTiming) {
		coTiming_ = coTiming;
	}

	public ArrayList<Integer> getTalkNum() {
		return talkNum_;
	}

	public void setTalkNum(ArrayList<Integer> talkNum) {
		talkNum_ = talkNum;
	}

	public ArrayList<Integer> getVotePattern() {
		return votePattern_;
	}

	public void setVotePattern(ArrayList<Integer> votePattern) {
		votePattern_ = votePattern;
	}

	public ArrayList<Integer> getResultBlack() {
		return resultBlack_;
	}

	public void setResultBlack(ArrayList<Integer> resultBlack) {
		resultBlack_ = resultBlack;
	}

	public Role getRole() {
		return role_;
	}

	public void setRole(Role role) {
		role_ = role;
	}


	public int getCODay() {
		int timing = getCoTiming();
		if (timing > 0) {
			return timing / 10;
		} else {
			return -1;
		}
	}

	public void checkSayHello(Utterance uttr) {
		sh_.getCheckWords(uttr);
	}

	public SayHello getSh() {
		return sh_;
	}

	public void setSh(SayHello sh) {
		sh_ = sh;
	}

}
