package com.gmail.sg06.greenrevolution.jingai_table;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.HashMap;

import org.aiwolf.common.data.Role;

import com.gmail.sg06.greenrevolution.data.COData;



public abstract class AbstractJingaiTable {

/**
 * tablemap  key：tableのインデックス，value:そのパターンの確率
 */
	private HashMap<Integer, Double> tableMap_;

/**
 * 倍率決定のマップ
 */
	private double[][] rateMap_;

	/**
	 * agents エージェント数　15
	 */
	public final int AGENT_NUM = 15;

	/**
	 * werewolfs 人狼数　3
	 */
	public final int WOLF_NUM = 3;

	/**
	 * possesses 狂人数　1
	 */
	public final int POS_NUM = 1;

	/**
	 * humans 村人陣営の人数　11
	 */
	public final int HUMAN_NUM = 11;

	/**
	 * combination 組合せの数
	 */
	private int combination_;

	/**
	 *table インデックス×エージェントの配列．0:村人, 1:人狼, 2:狂人
	 */
	private int table_[][];

	/**
	 * rates 倍率を更新する値の配列
	 */
	private double rates_[];


	/**
	 * コンストラクタ
	 */
	public AbstractJingaiTable(){
		//
		combination_=1;
//		System.out.println(combination_);
		for(int i = AGENT_NUM; i > HUMAN_NUM; i--){
			combination_*=i;
//			System.out.println(combination_);
		}
		for(int i = WOLF_NUM; i > 0; i--){
			combination_/=i;
//			System.out.println(combination_);
		}
		for(int i = POS_NUM; i > 0; i--){
			combination_/=i;
//			System.out.println(combination_);
		}
		tableMap_ = new HashMap<Integer, Double>();
		rateMap_  = new double[combination_][3];
		table_    = new int[combination_][AGENT_NUM];
		table_    = combination();

		//tablemapを初期化
		int l = getTable().length;
		for(int key = 0; key < l; key++){
			tableMap_.put(key, 1.0);
			rateMap_[key][0] = 1.0;
			rateMap_[key][1] = 1.0;
			rateMap_[key][2] = 1.0;
		}

		//ratesを初期化
		double[] defaultRates = {0.9, 0.9, 0.5, 0.9, 0.5, 0.1, 0.01, 0.01, 0.0, 0.01, 0.01, 0.1, 0.001, 0.001, 0.001};
		setRates(defaultRates);
	}


/**
 * 各テーブルの値を1.0 で初期化する.
 * isAllTable が false であれば, 倍率テーブルだけ更新する.
 *
 * @param isAllTable すべてのテーブルを初期化するか.
 */
	public void initTable(boolean isAllTable){
		for ( int key = 0; key < table_.length; key++ ){
			rateMap_[key][0] = 1.0;
			rateMap_[key][1] = 1.0;
			rateMap_[key][2] = 1.0;
		}
		if ( isAllTable ){
			for ( int key = 0; key < table_.length; key++ ){
				tableMap_.put(key, 1.0);
			}
		}
	}


/**
 * upType 番目の倍率テーブルの値を 1.0 で初期化する.
 *
 * @param upType
 *            更新する種類( 0: 襲撃・投票, 1: CO 関連・会話関連)
 */
	public void initRateTable(int upType){
		if ( (upType>-1) && (upType<rateMap_[0].length) ){
			for ( int key = 0; key < table_.length; key++ ){
				rateMap_[key][upType] = 1.0;
			}
		}
	}


	public void normalizeWolfSideTable(){
	// 最大値が 1.0 になるように正規化する
		double max = Double.NEGATIVE_INFINITY;
		for ( int i : tableMap_.keySet() ){
			if ( tableMap_.get(Integer.valueOf(i)) > max ){
				max = tableMap_.get(Integer.valueOf(i));
			}
		}
		if ( max != 0.0 ){
			for ( int i : tableMap_.keySet() ){
				double value = tableMap_.get(Integer.valueOf(i));
				tableMap_.put(Integer.valueOf(i), value/max);
			}
		}else{
			// 最大値が 0.0 ならば初期化する
			for ( int i : tableMap_.keySet() ){
				double value = tableMap_.get(Integer.valueOf(i));
				tableMap_.put(Integer.valueOf(i), 1.0);
			}
		}
	}



	/**
	 * 組合せ配列を作る
	 * @return
	 */
	private int[][] combination() {
		// TODO 自動生成されたメソッド・スタブ
		int table[][] = new int[combination_][AGENT_NUM];//返す配列


		int n = AGENT_NUM;
		int r = WOLF_NUM;
		Combination c = new Combination(n,r);//コンビネーション計算．15C3

		long csize = c.getSize();//組合せの数．15C3=455
//		System.out.println("size: " + csize + " combi: " + combination_);
		int l = 0;//0~5459 パターンのインデックス
		for(int i = 0; i < csize; i++){

			int[] wolfs = new int[r];//狼のインデックスの配列
			c.getElements(i, wolfs);
			for(int j = 0; j < n; j++){//狂人を選択
				boolean flag = true;
				for(int k: wolfs){
					if(j == k){//狂人と人狼がかぶっていたらだめ
						flag=false;
						break;

					}

				}

				if(flag== false) continue;
				//System.out.println(j + " " + wolfs[0] + " " + wolfs[1] + " " + wolfs[2]);
				for(int k = 0; k < n; k++){//人狼と狂人が被っていないなら，
//					System.out.println(l + " " + k + " " + j);
					if(k == j){//狂人は2
						table[l][k]=2;
					}
					else if(k == wolfs[0] || k == wolfs[1] || k == wolfs[2]){//人狼は1
						table[l][k]=1;
					}
					else table[l][k]=0;//村人陣営は0

				}
				l++;
			}

		}

		return table;
	}



	/**
	 * 倍率設定
	 * @param index 項目のインデックス
	 * @param newrate 変更後の値
	 */
	public void setRate(int index, double newrate){//
		rates_[index]=newrate;
	}


/**
 * 襲撃による人外テーブルの更新
 *
 * @param targetIdx 襲撃されたエージェントの番号
 */
	abstract public void calcAttackInfo(int targetIdx);


/**
 * 投票による人外テーブルの更新
 *
 * @param agentIdx 投票したエージェントの番号
 * @param targetIdx 投票されたエージェントの番号
 */
	abstract public void calcVoteInfo(int agentIdx, int targetIdx);


/**
 * 日数と投票による人外テーブルの更新
 *
 * @param agentIdx 投票したエージェントの番号
 * @param targetIdx 投票されたエージェントの番号
 * @param day 投票の日数
 */
	abstract public void calcVoteInfo(int agentIdx, int targetIdx, int day);


/**
 * 占い結果による人外テーブルの更新
 *
 * @param agentIdx 占ったエージェントの番号
 * @param targetIdx 占われたエージェントの番号
 * @param isBlack 占い結果が黒であるか
 */
	abstract public void calcDivineInfo(int agentIdx, int targetIdx, boolean isBlack);


/**
 * 霊能結果による人外テーブルの更新
 *
 * @param agentIdx 霊能したエージェントのエージェント番号
 * @param targetIdx 霊能されたエージェントのエージェント番号
 * @param isBlack 霊能結果が黒であるか
 */
	abstract public void calcInquestInfo(int agentIdx, int targetIdx, boolean isBlack);


	// TODO 狂人がCOしないパターンも実装したいが, COがあった毎には更新は無理？

/**
 * 占いCOによる人外テーブルの更新
 *
 * @param seerCOList 更新済みの占いCOリスト(エージェントは番号で管理)
 */
	abstract public void calcSeerCOInfo(ArrayList<Integer> seerCOList);


/**
 * 霊能COによる人外テーブルの更新
 *
 * @param seerCOList 更新済みの霊能COリスト(エージェントは番号で管理)
 */
	abstract public void calcMediumCOInfo(ArrayList<Integer> mediumCOList);


/**
 * 狩人COによる人外テーブルの更新
 *
 * @param seerCOList 更新済みの狩人COリスト(エージェントは番号で管理)
 */
	abstract public void calcBGCOInfo(ArrayList<Integer> BGCOList);


/**
 * COしていないことによる人外テーブルの更新
 *
 * @param notCOList
 */
	abstract public void calcNotCOInfo(ArrayList<Integer> notCOList);


/**
 * 占いと霊能の組合せの矛盾による人外テーブルの更新
 *
 * @param seerIdx 占い CO のエージェント番号
 * @param seerCOData 占い CO のこれまでの結果
 * @param mediumIdx  霊能 CO のエージェント番号
 * @param mediumCOData 霊能 CO のこれまでの結果
 */
	abstract public void calcPatternInfo(int seerIdx, COData seerCOData, int mediumIdx, COData mediumCOData);



/**
 * 倍率テーブルを用いて, 人外テーブルを一括で更新する.
 */
	public void update(){
		for ( int key = 0; key < table_.length; key++ ){
			for ( int i = 0; i < rateMap_[key].length; i++ ){
				double rate  = rateMap_[key][i];
				double table = tableMap_.get(key);
				tableMap_.put(key, table*rate);
			}
		}
	}








/**
 * attack, comingoutで使う
 * @param agentIndex エージェントのインデックス
 * @param role 役割{0:村人側, 1:人狼, 2:狂人}
 * @return
 */
/**
 * agentIdx の役割が role である人外テーブルのインデックスをリストで返す
 * @param agentIdx エージェント番号
 * @param role 役割
 * @return
 */
	public ArrayList<Integer> tableIndexList(int agentIdx, Role role){
		int r_num = 0;		// 役割番号
		// 役割番号：{村陣営 -> 0, 人狼 -> 1, 狂人 -> 2}}
		if ( role == Role.WEREWOLF ){
			r_num = 1;
		} else if ( role == Role.POSSESSED ){
			r_num = 2;
		}
		ArrayList<Integer> list = new ArrayList<Integer>();//agentIndexの役割がroleのtableのインデックスのリスト
		for(int i = 0; i < combination_; i++){
			if( table_[i][agentIdx-1] == r_num ){
				list.add(Integer.valueOf(i));
			}
		}

		return list;
	}


/**
 * agentIdxList.get(i) の役割が roleList.get(i) である人外テーブルのインデックスをリストで返す.
 * 全条件を満たしたものをリストに加える.
 * @param agentIdxList エージェント番号のリスト
 * @param roleList 役割のリスト
 * @return
 */
	public ArrayList<Integer> tableIndexList(ArrayList<Integer> agentIdxList, ArrayList<Role> roleList){
		ArrayList<Integer> list = new ArrayList<Integer>();
		// agentIdxList と roleList のサイズが違えば空のリストを返す
		if ( agentIdxList.size() != roleList.size() ){
			return list;
		}
		int[] r_nums = new int[roleList.size()];		// 役割番号
		// 役割番号：{村陣営 -> 0, 人狼 -> 1, 狂人 -> 2}}
		for ( int i = 0; i < roleList.size(); i++ ){
			Role role = roleList.get(i);
			r_nums[i] = 0;
			if ( role == Role.WEREWOLF ){
				r_nums[i] = 1;
			} else if ( role == Role.POSSESSED ){
				r_nums[i] = 2;
			}
		}
		for ( int i = 0; i < combination_; i++ ){
			boolean allClear = true;
			for ( int j = 0; j < roleList.size(); j++ ){
				if (table_[i][agentIdxList.get(j)-1] != r_nums[j] ){
					allClear = false;
					break;
				}
			}
			if ( allClear ){
				list.add(i);
			}
		}

		return list;
	}
/**
 *
 * @param agentIndexFrom 主体エージェントのインデックス
 * @param roleFrom 主体の役割{0:村人側, 1:人狼, 2:狂人}
 * @param agentIndexTo 客体エージェントのインデックス
 * @param roleTo 客体の役割{0:村人側, 1:人狼, 2:狂人}
 * @return
 */
	public ArrayList<Integer> tableIndexList(int agentIndexFrom, Role roleFrom, int agentIndexTo, Role roleTo){//vote,judgeで使う
		int r_from_num=0,r_to_num=0;

		if(roleFrom == Role.WEREWOLF) r_from_num=1;
		else if(roleFrom == Role.POSSESSED) r_from_num=2;

		if(roleTo == Role.WEREWOLF) r_to_num=1;
		else if(roleTo == Role.POSSESSED) r_to_num=2;

		ArrayList<Integer> list = new ArrayList<Integer>();
		for(int i = 0; i < 5460; i++){
			if ( (table_[i][agentIndexFrom-1]==r_from_num) && (table_[i][agentIndexTo-1]==r_to_num) ){
				list.add(Integer.valueOf(i));
			}
		}
		return list;
	}

	public static void main(String[] args) {
		// TODO 自動生成されたメソッド・スタブ
		int combination_ = 5460;
		int AGENT_NUM = 15;
		int WOLF_NUM=3;

		int table[][] = new int[combination_][AGENT_NUM];//返す配列

		int n = AGENT_NUM;
		int r = WOLF_NUM;
		Combination c = new Combination(n,r);//コンビネーション計算．15C3

		long csize = c.getSize();//組合せの数．15C3=455
		System.out.println("size: " + csize);
		int l = 0;//0~5459 パターンのインデックス
		for(int i = 0; i < csize; i++){

			int[] wolfs = new int[r];//狼のインデックスの配列
			c.getElements(i, wolfs);
/*			String txt = "";
			for(int k: wolfs){
				txt+= k + ", ";
			}
			txt+="\n";
			System.out.println(txt);
*/
			for(int j = 0; j < n; j++){//狂人を選択
				boolean flag = true;
				for(int k: wolfs){
					if(j == k){//狂人と人狼がかぶっていたらだめ
						flag=false;
						break;

					}

				}

				if(flag== false) continue;
				//System.out.println(j + " " + wolfs[0] + " " + wolfs[1] + " " + wolfs[2]);
				for(int k = 0; k < n; k++){//人狼と狂人が被っていないなら，
					if(k == j){//狂人は2
						table[l][k]=2;
					}
					else if(k == wolfs[0] || k == wolfs[1] || k == wolfs[2]){//人狼は1
						table[l][k]=1;
					}
					else table[l][k]=0;//村人陣営は0

				}
				l++;
			}

		}
		String str="";
		for(int i = 0; i < combination_; i++){
			str+="{";
			for(int j =0; j < AGENT_NUM; j++){
				str += table[i][j] + ", ";
			}
			str += "},\n";
		}
		System.out.println(str);
		File file = new File("combination.txt");
		FileWriter filewriter = null;
		try {
			filewriter = new FileWriter(file);
		} catch (IOException e) {
			// TODO 自動生成された catch ブロック
			e.printStackTrace();
		}
		BufferedWriter bw = new BufferedWriter(filewriter);
		PrintWriter pw = new PrintWriter(bw);
		pw.print("{" + str + "}");

	}


	/**
	 * tableMap_のゲッター
	 * @return
	 */
	public HashMap<Integer, Double> getTablemap() {
		return tableMap_;
	}

	/**
	 * table_のゲッター
	 * @return
	 */
	public int[][] getTable(){
		return table_;
	}

	/**
	 * table_のセッター
	 * @return
	 */
	public void setTable(int[][] newTable){
		table_ = newTable;
	}

	/**
	 * rates_のゲッター
	 * @return
	 */
	public double getRate(int num){
		return rates_[num];
	}
	public double[] getRates(){
		return rates_;
	}

	/**
	 * rates_のセッター
	 * @return
	 */
	public void setRate(double newRate, int num){
		rates_[num] = newRate;
	}
	public void setRates(double[] newRates){
		rates_ = newRates;
	}

	public double[][] getRateTable(){
		return rateMap_;
	}




	/**
	 * あるエージェントが人狼である確率を返す
	 *
	 * @param agentIndex エージェント番号
	 * @return
	 */
	public double getWolfRate(int agentIndex){
		ArrayList<Integer> list = tableIndexList(agentIndex, Role.WEREWOLF);
		double a=0;
		for(int i: list){
			a+=tableMap_.get(i);
		}
		double b=0;
		for(int i = 0; i < combination_; i++){
//			System.out.println(i + ":" + tableMap_.get(i));
			b+= tableMap_.get(i);
		}
//		System.out.println("wolfrate-" + agentIndex + ":" + a + "/" + b + "=" + (a/b));
//		if ( b == 0 ){
//			System.exit(-1);
//		}
		return a/b;
	}

	/**
	 * あるエージェントが人狼でない確率を返す
	 *
	 * @param agentIndex エージェント番号
	 * @return
	 */
	public double getNotWolfRate(int agentIndex){
		ArrayList<Integer> list = tableIndexList(agentIndex, Role.VILLAGER);
		double a=0;
		for(int i: list){
			a+=tableMap_.get(i);
		}

		double b=0;
		for(int i = 0; i < combination_; i++){
			b+= tableMap_.get(i);
		}
//		System.out.println("notwolfrate-" + agentIndex + ":" + a + "/" + b);
		return a/b;
	}



}
