package com.gmail.sg06.greenrevolution.weightedplayer;

import java.util.ArrayList;
import java.util.Random;

import org.aiwolf.client.base.player.AbstractRole;
import org.aiwolf.client.lib.TemplateTalkFactory;
import org.aiwolf.client.lib.TemplateWhisperFactory;
import org.aiwolf.common.data.Agent;
import org.aiwolf.common.data.Role;
import org.aiwolf.common.net.GameInfo;
import org.aiwolf.common.net.GameSetting;

public class WeightedPlayer extends AbstractRole {

	ArrayList<AbstractRole> roleList_;

	private int seed_ = 1;

	double[][] weight_;

	private int roleSize_;

	private boolean isSay_ = false;

	private SayHello sayHello_ = new SayHello();

	public WeightedPlayer(ArrayList<AbstractRole> list, double[][] weight) {
		roleList_ = list;
		roleSize_ = list.size();
		weight_ = weight;
	}

	public WeightedPlayer(ArrayList<AbstractRole> list) {
		roleList_ = list;
		roleSize_ = list.size();
		double[][] weight = new double[6][roleSize_];
		for (int i = 0; i < weight.length; i++) {
			for (int j = 0; j < roleSize_; j++) {
				weight[i][j] = 1.0;
			}
		}
		weight_ = weight;
	}

	public WeightedPlayer(ArrayList<AbstractRole> list, boolean isRand) {
		Random r = new Random(seed_);
		r.nextBoolean();
		roleList_ = list;
		roleSize_ = list.size();
		double[][] weight = new double[6][roleSize_];
		for (int i = 0; i < weight.length; i++) {
			for (int j = 0; j < roleSize_; j++) {
				weight[i][j] = 2.0 * (r.nextDouble() - 0.5);
			}
		}
		// System.out.println(Arrays.deepToString(weight));
		WeightManager.setWeight(weight);
		weight_ = weight;
	}

	public void initialize(GameInfo gameInfo, GameSetting gameSetting) {
		super.initialize(gameInfo, gameSetting);
	/*	for (int i = 0; i < roleSize_; i++) {
			if (roleList_.get(i) instanceof org.aiwolf.Satsuki.LearningPlayer.AbstractKajiBasePlayer
					|| roleList_.get(i) instanceof com.gmail.jinro.noppo.players.PlayerBase) {
				Class tmp = roleList_.get(i).getClass();
				AbstractRole tmp2;
				try {
					tmp2 = (AbstractRole) tmp.newInstance();
					roleList_.set(i, tmp2);
				} catch (InstantiationException | IllegalAccessException e) {
					e.printStackTrace();
				}
			}
		}
*/
		for (int i = 0; i < roleSize_; i++) {
			roleList_.get(i).initialize(gameInfo, gameSetting);
		}

		for (int i = 0; i < roleSize_; i++) {
			roleList_.get(i).initialize(gameInfo, gameSetting);
		}
		weight_ = WeightManager.getWeight();
	}

	public void update(GameInfo gameInfo) {
		super.update(gameInfo);
		for (int i = 0; i < roleSize_; i++) {
			roleList_.get(i).update(gameInfo);
		}

	}

	/**
	 *
	 * @param num
	 *            0: attack, 1: divine, 2: guard, 3: talk, 4: vote, 5: whisper
	 * @return
	 */
	private double[] getWeights(int num) {
		return weight_[num];
	}

	@Override
	public Agent attack() {
		double[] point = new double[15];
		for ( int i = 0; i < point.length; i++ ){
			point[i] = Double.NEGATIVE_INFINITY;
		}
		double randP = 0;
		double[] weight = getWeights(0);
		for (int i = 0; i < roleSize_; i++) {
			Agent target = roleList_.get(i).attack();
			if (target == null) {
				randP += weight[i];
				continue;
			}
			int index = target.getAgentIdx() - 1;
			if ( point[index] == Double.NEGATIVE_INFINITY ){
				point[index] = weight[i];
			}else{
				point[index] += weight[i];
			}
		}
		double max = Double.NEGATIVE_INFINITY;
		int maxIndex = 0;
		for (int i = 0; i < point.length; i++) {
			if (point[i] > max) {
				max = point[i];
				maxIndex = i;
			}
		}
		if (randP > max) {
			return null;
		} else {
			return getGameInfoMap().get(Integer.valueOf(this.getDay()))
					.getAgentList().get(maxIndex);
		}
	}

	@Override
	public void dayStart() {
		for (int i = 0; i < roleSize_; i++) {
			roleList_.get(i).dayStart();
		}
	}

	@Override
	public Agent divine() {
		double[] point = new double[15];
		for ( int i = 0; i < point.length; i++ ){
			point[i] = Double.NEGATIVE_INFINITY;
		}
		double[] weight = getWeights(1);
		for (int i = 0; i < roleSize_; i++) {
			Agent target = roleList_.get(i).divine();
			if (target == null) {
				System.out.println("divine null in agent:" + roleList_.get(i));
				continue;
			}
			int index = target.getAgentIdx() - 1;
			if ( point[index] == Double.NEGATIVE_INFINITY ){
				point[index] = weight[i];
			}else{
				point[index] += weight[i];
			}
		}
		double max = Double.NEGATIVE_INFINITY;
		int maxIndex = 0;
		for (int i = 0; i < point.length; i++) {
//			System.out.print("point[" + i + "]=" + point[i] + " , ");
			if (point[i] > max) {
				max = point[i];
				maxIndex = i;
			}
		}
//		System.out.println();
		return getGameInfoMap().get(Integer.valueOf(this.getDay()))
				.getAgentList().get(maxIndex);
	}

	@Override
	public void finish() {
		for (int i = 0; i < roleSize_; i++) {
			roleList_.get(i).finish();
		}
		isSay_ = true;
	}

	@Override
	public Agent guard() {
		double[] point = new double[15];
		for ( int i = 0; i < point.length; i++ ){
			point[i] = Double.NEGATIVE_INFINITY;
		}
		double[] weight = getWeights(2);
		for (int i = 0; i < roleSize_; i++) {
			Agent target = roleList_.get(i).guard();
			if (target == null) {
				System.out.println("guard null in agent:" + roleList_.get(i));
				continue;
			}
			int index = target.getAgentIdx() - 1;
			if ( point[index] == Double.NEGATIVE_INFINITY ){
				point[index] = weight[i];
			}else{
				point[index] += weight[i];
			}
		}
		double max = Double.NEGATIVE_INFINITY;
		int maxIndex = 0;
		for (int i = 0; i < point.length; i++) {
			if (point[i] > max) {
				max = point[i];
				maxIndex = i;
			}
		}
		return getGameInfoMap().get(Integer.valueOf(this.getDay()))
				.getAgentList().get(maxIndex);
	}

	@Override
	public String talk() {

		if (!isSay_ && getLatestDayGameInfo().getDay() == 0) {
			String[] words = sayHello_.getWords().split(":");
			if (words[0].equals("vote")) {
				if (words.length == 2) {
					int targetIndex = Integer.valueOf(words[1]);
					return TemplateTalkFactory.vote(getLatestDayGameInfo()
							.getAgentList().get(targetIndex));
				} else {
					System.err.println("error: WeightedPlayer");
				}
			}
			if (words[0].equals("skip")) {
				return TemplateTalkFactory.skip();
			}
			if (words[0].equals("estimate")) {
				if (words.length == 3) {
					int targetIndex = Integer.valueOf(words[1]);
					Agent target = getLatestDayGameInfo().getAgentList().get(
							targetIndex);
					Role role = transRole(words[2]);
					return TemplateTalkFactory.estimate(target, role);
				} else {
					System.err.println("error: WeightedPlayer");
				}
			}
			if (words[0].equals("you")) {
				if (words.length == 2) {
					int targetIndex = Integer.valueOf(words[1]);
					Agent target = getLatestDayGameInfo().getAgentList().get(
							targetIndex);
					Role role = getLatestDayGameInfo().getRole();
					return TemplateTalkFactory.estimate(target, role);
				} else {
					System.err.println("error: WeightedPlayer");
				}
			} else {
				System.err.println("error: WeightedPlayer");
			}
		}

		double max = Double.NEGATIVE_INFINITY;
		int maxIndex = 0;
		double[] weight = getWeights(3);
		for (int i = 0; i < roleSize_; i++) {
			if (weight[i] > max) {
				max = weight[i];
				maxIndex = i;
			}
		}
		String talk = roleList_.get(maxIndex).talk();

		for (int i = 0; i < roleSize_; i++) {
			if (i != maxIndex) {
				roleList_.get(i).talk();
			}
		}

		if (talk != null) {
			if (!talk.equals(TemplateTalkFactory.over())) {
				return talk;
			}
		}
		return TemplateTalkFactory.skip();
	}

	@Override
	public Agent vote() {
		double[] point = new double[15];
		for ( int i = 0; i < point.length; i++ ){
			point[i] = Double.NEGATIVE_INFINITY;
		}
		double randP = 0;
		double[] weight = getWeights(4);
		for (int i = 0; i < roleSize_; i++) {
			Agent target = roleList_.get(i).vote();
			if (target == null) {
				randP += weight[i];
				continue;
			}
			int index = target.getAgentIdx() - 1;
			if ( point[index] == Double.NEGATIVE_INFINITY ){
				point[index] = weight[i];
			}else{
				point[index] += weight[i];
			}
		}
		double max = Double.NEGATIVE_INFINITY;
		int maxIndex = 0;
		for (int i = 0; i < point.length; i++) {
			if (point[i] > max) {
				max = point[i];
				maxIndex = i;
			}
		}

		if (randP > max) {
			return null;
		} else {
			return getGameInfoMap().get(Integer.valueOf(this.getDay()))
					.getAgentList().get(maxIndex);
		}
	}

	@Override
	public String whisper() {
		double max = Double.NEGATIVE_INFINITY;
		int maxIndex = 0;
		double[] weight = getWeights(5);
		for (int i = 0; i < roleSize_; i++) {
			if (weight[i] > max) {
				max = weight[i];
				maxIndex = i;
			}
		}
		String whisper = roleList_.get(maxIndex).whisper();

		for (int i = 0; i < roleSize_; i++) {
			if (i != maxIndex) {
				roleList_.get(i).whisper();
			}
		}

		if (whisper != null) {
			if (!whisper.equals(TemplateWhisperFactory.over())) {
				return whisper;
			}
		}
		return TemplateWhisperFactory.skip();

	}

	public double[][] getWeight() {
		return weight_;
	}

	public void setWeight(double[][] weight) {
		weight_ = weight;
	}

	public void setWeight(int num, double[] weight) {
		weight_[num] = weight;
	}

	private Role transRole(String roleName) {
		if (roleName.equals("v")) {
			return Role.VILLAGER;
		}
		if (roleName.equals("w")) {
			return Role.WEREWOLF;
		}
		if (roleName.equals("m")) {
			return Role.MEDIUM;
		}
		if (roleName.equals("b")) {
			return Role.BODYGUARD;
		}
		if (roleName.equals("s")) {
			return Role.SEER;
		}
		return Role.POSSESSED;
	}
}
